@@ -187,8 +187,8 @@ void check_format_options(void)
 		flags &= ~S_F_HORIZONTALLY;
 	}
 	if (!ACCEPT_LOCAL_TIME(fmt[f_position]->options)) {
-		/* Remove options -T and -t */
-		flags &= ~(S_F_LOCAL_TIME + S_F_TRUE_TIME);
+		/* Remove option -T */
+		flags &= ~S_F_LOCAL_TIME;
 	}
 	if (!ACCEPT_SEC_EPOCH(fmt[f_position]->options)) {
 		/* Remove option -U */
@@ -717,13 +717,13 @@ int generic_write_stats(int curr, int use_tm_start, int use_tm_end, int reset,
 			continue;
 
 		if ((TEST_MARKUP(fmt[f_position]->options) && CLOSE_MARKUP(act[i]->options)) ||
-		    (IS_SELECTED(act[i]->options) && (act[i]->nr > 0))) {
+		    (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0))) {
 
 			if (format == F_JSON_OUTPUT) {
 				/* JSON output */
 				int *tab = (int *) parm;
 
-				if (IS_SELECTED(act[i]->options) && (act[i]->nr > 0)) {
+				if (IS_SELECTED(act[i]->options) && (act[i]->nr[curr] > 0)) {
 
 					if (*fmt[f_position]->f_timestamp) {
 						(*fmt[f_position]->f_timestamp)(tab, F_MAIN, cur_date, cur_time,
@@ -958,7 +958,7 @@ void display_curr_act_graphs(int ifd, int *curr, long *cnt, int *eosaf,
 			*curr ^= 1;
 		}
 	}
-	while (!*eosaf);
+	while (!*eosaf && *cnt);
 
 	*reset = TRUE;
 
@@ -1816,7 +1816,7 @@ int main(int argc, char **argv)
 				usage(argv[0]);
 			}
 			interval = atol(argv[opt++]);
-			if (interval <= 0) {
+			if (interval < 1) {
 				usage(argv[0]);
 			}
 		}
@@ -1866,7 +1866,7 @@ int main(int argc, char **argv)
 		tm_end.tm_hour += 24;
 	}
 
-	if (USE_PRETTY_OPTION(flags)) {
+	if (DISPLAY_PRETTY(flags)) {
 		dm_major = get_devmap_major();
 	}
 
